@@ -7,7 +7,7 @@
 ;; URL: https://git.sr.ht/~yantar92/emacs-gc-stats
 ;; Package-Requires: ((emacs "25.1"))
 
-;; Version: 1.2.3
+;; Version: 1.3
 
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -54,12 +54,29 @@
   :type 'file)
 
 (defcustom emacs-gc-stats-gc-defaults nil
-  "GC strategy to be active in `emacs-gc-stats-mode'."
+  "GC strategy to be active in `emacs-gc-stats-mode'.
+This setting, when non-nil, will override the existing values of
+`gc-cons-threshold' and `gc-cons-percentage'."
   :type '(choice
 	  (const :tag "Do not change existing GC settings" nil)
           (const :tag "Force emacs defaults" emacs-defaults)))
 
-(defvar emacs-gc-stats--setting-vars
+(defcustom emacs-gc-stats-inhibit-command-name-logging nil
+  "When non-nil, do not collect command names (`this-command')."
+  :type 'boolean
+  :package-version '(emacs-gc-stats . 1.3))
+
+(defcustom emacs-gc-stats-remind nil
+  "When non-nil, remind about submitting the data to Emacs devs.
+The value is wither nil (do not remind), t (remind in 3 weeks), or a
+number of days."
+  :type '(choice
+	  (const :tag "No reminder" nil)
+          (const :tag "Remind in 3 weeks" t)
+          (integer :tag "Remind after N days"))
+  :package-version '(emacs-gc-stats . 1.3))
+
+(defcustom emacs-gc-stats-setting-vars
   '(gc-cons-threshold
     gc-cons-percentage
     memory-limit
@@ -69,9 +86,11 @@
     prelude-tips
     (memory-info)
     (memory-use-counts))
-  "List of variable/function symbols to collect.")
+  "List of variable/function symbols to collect after loading init.el."
+  :type '(list sexp)
+  :package-version '(emacs-gc-stats . 1.3))
 
-(defvar emacs-gc-stats--command-vars
+(defcustom emacs-gc-stats-command-vars
   '(gc-cons-threshold
     gc-cons-percentage
     gcmh-mode
@@ -82,9 +101,11 @@
     (memory-info)
     (memory-use-counts)
     emacs-gc-stats--idle-tic)
-  "List of variable/function symbols to collect for each GC or command.")
+  "List of variable/function symbols to collect for each GC or command."
+  :type '(list sexp)
+  :package-version '(emacs-gc-stats . 1.3))
 
-(defvar emacs-gc-stats--summary-vars
+(defcustom emacs-gc-stats-summary-vars
   '(gc-cons-threshold
     gc-cons-percentage
     gc-elapsed
@@ -95,7 +116,9 @@
     (memory-use-counts)
     emacs-gc-stats-idle-delay
     emacs-gc-stats--idle-tic)
-  "List of variables to collect at session end.")
+  "List of variables to collect at session end."
+  :type '(list sexp)
+  :package-version '(emacs-gc-stats . 1.3))
 
 (defun emacs-gc-stats--collect (&rest symbols)
   "Collect SYMBOLS values.
@@ -108,7 +131,10 @@ Otherwise, collect symbol."
       (pcase var
         ((pred keywordp) (push var data))
         ((and (pred symbolp) (pred boundp))
-         (push (cons var (symbol-value var)) data))
+         (unless (and emacs-gc-stats-inhibit-command-name-logging
+		      (memq var '( this-command real-this-command
+				   last-command real-last-command)))
+           (push (cons var (symbol-value var)) data)))
         ((pred functionp)
          (push (cons var (funcall var)) data))
         ((pred listp)
@@ -125,7 +151,7 @@ Otherwise, collect symbol."
    (apply #'emacs-gc-stats--collect
           "Initial stats"
           (current-time-string)
-          emacs-gc-stats--setting-vars)
+          emacs-gc-stats-setting-vars)
    emacs-gc-stats--data))
 
 (defun emacs-gc-stats--collect-gc ()
@@ -133,7 +159,7 @@ Otherwise, collect symbol."
   (push
    (apply #'emacs-gc-stats--collect
           (current-time-string)
-          emacs-gc-stats--command-vars)
+          emacs-gc-stats-command-vars)
    emacs-gc-stats--data))
 
 (defun emacs-gc-stats--collect-init-end ()
@@ -142,7 +168,7 @@ Otherwise, collect symbol."
    (apply #'emacs-gc-stats--collect
           "Init.el stats"
           (current-time-string)
-          emacs-gc-stats--summary-vars)
+          emacs-gc-stats-summary-vars)
    emacs-gc-stats--data))
 
 (defun emacs-gc-stats--collect-end ()
@@ -151,18 +177,21 @@ Otherwise, collect symbol."
    (apply #'emacs-gc-stats--collect
           "Session end stats"
           (current-time-string)
-          emacs-gc-stats--summary-vars)
+          emacs-gc-stats-summary-vars)
    emacs-gc-stats--data))
 
+(defun emacs-gc-stats--get-repvious-session-data ()
+  "Return previously saved session data."
+  (and (file-readable-p emacs-gc-stats-file)
+       (with-temp-buffer
+         (insert-file-contents emacs-gc-stats-file)
+         (ignore-errors (read (current-buffer))))))
+
 (defun emacs-gc-stats-save-session ()
   "Save stats to disk."
   (interactive)
   (emacs-gc-stats--collect-end)
-  (let ((previous-sessions
-         (and (file-readable-p emacs-gc-stats-file)
-              (with-temp-buffer
-                (insert-file-contents emacs-gc-stats-file)
-                (ignore-errors (read (current-buffer))))))
+  (let ((previous-sessions (emacs-gc-stats--get-repvious-session-data))
         (session (reverse emacs-gc-stats--data))
         (write-region-inhibit-fsync t)
         ;; We set UTF-8 here to avoid the overhead from
@@ -185,12 +214,13 @@ Otherwise, collect symbol."
           (push session previous-sessions)))
       (prin1 previous-sessions (current-buffer)))
     (when
-	(and (called-interactively-p)
+	(and (called-interactively-p 'interactive)
 	     (yes-or-no-p
 	      (format "GC stats saved to \"%s\".  Send email to emacs-gc-stats@gnu.org? " emacs-gc-stats-file)))
       (browse-url "mailto:emacs-gc-stats@gnu.org"))
     (message "GC stats saved to \"%s\".  You can share the file by sending email to emacs-gc-stats@gnu.org" emacs-gc-stats-file)))
 
+(defvar emacs-gc-stats-mode) ; defined later
 (defun emacs-gc-stats-clear ()
   "Clear GC stats collected so far."
   (interactive)
@@ -204,7 +234,8 @@ Otherwise, collect symbol."
   "Delay in seconds to count idle time.")
 
 (defvar emacs-gc-stats--idle-tic 0
-  "Idle counter.")
+  "Idle counter.
+Idle time is counted with `emacs-gc-stats-idle-delay' granularity.")
 (defvar emacs-gc-stats--idle-timer nil
   "Time counting idle time.")
 (defun emacs-gc-stats-idle-tic ()
@@ -232,6 +263,35 @@ Revert original settings when RESTORE is non-nil."
 	     gc-cons-percentage 0.1))
       (other (error "Unknown value of `emacs-gc-stats-gc-defaults': %S" other)))))
 
+(defun emacs-gc-stats--remind-maybe ()
+  "Show a reminder according to `emacs-gc-stats-remind'."
+  (require 'notifications)
+  (when emacs-gc-stats-remind
+    (when-let* ((days-threshold (if (numberp emacs-gc-stats-remind)
+				    emacs-gc-stats-remind 21))
+		(first-session (or (car (last (emacs-gc-stats--get-repvious-session-data)))
+				   (reverse emacs-gc-stats--data)))
+		(first-record (car first-session))
+                (first-date-string
+                 (if (equal "Initial stats" (car first-record))
+                     (cadr first-record) (car first-record)))
+                (first-time (parse-time-string first-date-string))
+                (days-passed (time-to-number-of-days
+			      (time-subtract (current-time)
+					     (encode-time first-time)))))
+      (when (> days-passed days-threshold)
+        (notifications-notify
+         :title "emacs-gc-stats reminder"
+         :body
+         (format
+          "%.1f days have passed since first record.
+Consider M-x emacs-gc-stats-save-session or reporting back to emacs-gc-stats@gnu.org"
+	  days-passed))
+        (warn
+         "emacs-gc-stats: %.1f days have passed since first record.
+Consider M-x emacs-gc-stats-save-session or reporting back to emacs-gc-stats@gnu.org"
+	 days-passed)))))
+
 ;;;###autoload
 (define-minor-mode emacs-gc-stats-mode
   "Toggle collecting Emacs GC statistics."
@@ -240,6 +300,8 @@ Revert original settings when RESTORE is non-nil."
       (progn
 	(emacs-gc-stats--set-gc-defaults)
         (add-hook 'after-init-hook #'emacs-gc-stats--set-gc-defaults)
+        (add-hook 'after-init-hook #'emacs-gc-stats--remind-maybe)
+        (add-hook 'kill-emacs-hook #'emacs-gc-stats--remind-maybe)
         (unless emacs-gc-stats--data
           (emacs-gc-stats--collect-init))
         ;; 5 minutes counter.
@@ -252,6 +314,8 @@ Revert original settings when RESTORE is non-nil."
         (add-hook 'after-init-hook #'emacs-gc-stats--collect-init-end)
         (add-hook 'kill-emacs-hook #'emacs-gc-stats-save-session))
     (remove-hook 'after-init-hook #'emacs-gc-stats--set-gc-defaults)
+    (remove-hook 'after-init-hook #'emacs-gc-stats--remind-maybe)
+    (remove-hook 'kill-emacs-hook #'emacs-gc-stats--remind-maybe)
     (emacs-gc-stats--set-gc-defaults 'restore)
     (when (timerp emacs-gc-stats--idle-timer)
       (cancel-timer emacs-gc-stats--idle-timer))
